#ifndef P_TRANSFORMCENTER
#define P_TRANSFORMCENTER

#include "PixelCallback.h"
#include "../global.h"

typedef struct TransformCenter {
	i32 centerX;
	i32 centerY;
} TransformCenter;

PixelCallback* TransformCenter_setup(TransformCenter obj);

#endif /* P_TRANSFORMCENTER */
