#ifndef F_FRAME_RANGE
#define F_FRAME_RANGE

#include "../global.h"
#include "../FrameRender/PixelCallback.h"
#include "FrameCallback.h"

typedef struct FrameRange {
	u32 startFrame;
	u32 endFrame;
} FrameRange;

FrameCallback* FrameRange_setup(FrameRange obj);

#endif /* F_FRAME_RANGE */
